using System;
using Autofac;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FitConnect.Services.Models.v1.Submission;
using FluentAssertions;
using MockContainer;
using NUnit.Framework;
using SecurityEventToken = FitConnect.Models.SecurityEventToken;

namespace BasicUnitTest;

[TestFixture]
public class SecurityEventTokenTests {
    [SetUp]
    public void Setup() {
        var container = Container.Create();
        _encryption = new FitEncryption(container.Resolve<KeySet>(), null);
    }

    private const string rejectSubmission =
        SecurityEventToken.RejectSubmissionSchema;

    private FitEncryption _encryption = null!;

    private const string acceptSubmission =
        SecurityEventToken.AcceptSubmissionSchema;


    [Test]
    public void CreateJwt_AcceptSubmission() {
        var token = _encryption.CreateAcceptSecurityEventToken(new SubmissionForPickupDto {
            Id = Guid.NewGuid().ToString(), CaseId = Guid.NewGuid().ToString(),
            DestinationId = Guid.NewGuid().ToString()
        }).Result;
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(acceptSubmission);
        decoded.EventType.Should().Be(EventType.Accept);
    }

    [Test]
    public void CreateJwt_Reject_WithEncryptionIssue() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { new Problems(Problems.ProblemTypeEnum.EncryptionIssue, "UnitTest") }
        ).Result;
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithMissingSchema() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { new Problems(Problems.ProblemTypeEnum.MissingSchema, "UnitTest") }
        ).Result;
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);

        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithSchemaViolation() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { new Problems(Problems.ProblemTypeEnum.SchemaViolation, "UnitTest") }
        ).Result;
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithSyntaxViolation() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { new Problems(Problems.ProblemTypeEnum.SyntaxViolation, "UnitTest") }
        ).Result;
        Console.WriteLine(token);

        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithUnsupportedSchema() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { new Problems(Problems.ProblemTypeEnum.UnsupportedDataSchema, "UnitTest") }
        ).Result;
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithIncorrectAuthenticationTag() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { new Problems(Problems.ProblemTypeEnum.IncorrectAuthenticationTag, "UnitTest") }
        ).Result;
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_WithEncryptionKey_ShouldWork() {
        _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(), "DummyEvent", null).Wait();
    }

    [Test]
    public void CreateJwt_WithEncryptionKey_ShouldThrowAnException() {
        var ae = Assert.Throws<AggregateException>(() =>
            new FitEncryption(_encryption.PrivateKeyDecryption,
                    _encryption.PublicKeyEncryption, // using wrong key
                    null) {
                    PublicKeyEncryption = _encryption.PublicKeyEncryption,
                    PublicKeySignatureVerification = _encryption.PublicKeySignatureVerification
                }
                .CreateSecurityEventToken(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(),
                    Guid.NewGuid().ToString(), "DummyEvent", null).Wait()
        );
        Assert.AreEqual(
            typeof(InvalidOperationException), ae!.InnerException!.GetType(),
            "There was no InvalidOperationException.");
    }
}
