using System.Reflection;
using System.Security.Cryptography.X509Certificates;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;

namespace FitConnect.Encryption;

public class CertificateHelper {
    private readonly ILogger? _logger;

    public CertificateHelper(ILogger? logger = null) {
        _logger = logger;
    }

    internal async Task<bool> ValidateCertificateJsonWebKeyString(string keyJson, LogLevel logLevel,
        X509Certificate2[]? rootCertificates = null) {
        return await ValidateCertificateJsonWebKeyAsync(new JsonWebKey(keyJson), rootCertificates,
            logLevel);
    }

    internal static async Task<IList<X509Certificate2>> GetRootCertificates() {
        var result = new List<X509Certificate2>();
        var assembly = Assembly.GetExecutingAssembly();
        foreach (var resourceName in assembly.GetManifestResourceNames()
                     .Where(s =>
                         s.EndsWith(".pem") && s.Contains("FitConnect.TrustedRootCertificates."))) {
            var stream = assembly.GetManifestResourceStream(resourceName);
            if (stream == null) continue;
            var reader = new StreamReader(stream);
            var content = await reader.ReadToEndAsync();
            result.Add(X509Certificate2.CreateFromPem(content));
        }

        return result;
    }


    internal async Task<bool> ValidateCertificatePemAsync(X509Certificate2 certificate,
        X509Certificate2[]? rootCertificates = null,
        X509Certificate2[]? extras = null,
        LogLevel logLevel = LogLevel.Warning) {
        rootCertificates ??= (await GetRootCertificates()).ToArray();

        var certificateChain = new X509Chain();
        if (extras != null)
            certificateChain.ChainPolicy.ExtraStore.AddRange(extras);

        _logger?.LogDebug("Issuers: {Issuer}", certificate.Issuer);

        certificateChain.ChainPolicy.RevocationMode = X509RevocationMode.Online;
        certificateChain.ChainPolicy.RevocationFlag = X509RevocationFlag.EntireChain;
        certificateChain.ChainPolicy.TrustMode = X509ChainTrustMode.CustomRootTrust;
        certificateChain.ChainPolicy.CustomTrustStore.AddRange(rootCertificates);
        certificateChain.ChainPolicy.VerificationFlags = X509VerificationFlags.NoFlag;

        var result = certificateChain.Build(certificate);

        var statusAggregation = certificateChain.ChainStatus.Aggregate("",
            (r, s) => r + "\n\t - " + s.Status + ": " + s.StatusInformation);

        if (certificateChain.ChainStatus.Length > 0)
            _logger?.Log(result ? LogLevel.Information : logLevel,
                "Certificate status: {ObjStatusInformation}",
                statusAggregation);

        return result;
    }

    internal async Task<bool> ValidateCertificateJsonWebKeySet(JsonWebKeySet keys,
        X509Certificate2[]? rootCertificates = null,
        LogLevel logLevel = LogLevel.Error) {
        if (keys.Keys.Count == 0) {
            _logger?.Log(logLevel, "No keys found");
            return false;
        }

        var result = true;
        foreach (var key in keys.Keys)
            result = result &&
                     await ValidateCertificateJsonWebKeyAsync(key, rootCertificates, logLevel);
        return result;
    }

    internal async Task<bool> ValidateCertificateJsonWebKeyAsync(JsonWebKey key,
        X509Certificate2[]? root = null,
        LogLevel logLevel = LogLevel.Error) {
        var certificates = key.X5c.Select(s => new X509Certificate2(Convert.FromBase64String(s)))
            .ToList();

        if (certificates.Count == 0) {
            _logger?.Log(logLevel, "No certificates found in key");
            return false;
        }

        var fitConnectRequirements = key.MatchesFitConnectRequirements();
        if (!fitConnectRequirements) {
            _logger?.Log(logLevel, "Certificate does not match FIT-Connect requirements");
            return false;
        }

        var valid = await ValidateCertificatePemAsync(certificates.First(), root, certificates.ToArray(),
            logLevel);
        return valid && fitConnectRequirements;
    }
}

public static class CertificateExtensions {
    public static bool MatchesFitConnectRequirements(this JsonWebKey key) {
        return key.X5c.Count == 3
               && key.KeySize == 4096
               && (key.Alg == "PS512" || key.Alg == "RSA-OAEP-256")
               && (key.KeyOps.Contains("wrapKey") || key.KeyOps.Contains("verify"));
    }
}
