#nullable disable
using Newtonsoft.Json;

namespace FitConnect.Models.v1.Api;

public class Problems {
    // @formatter:off
    public const string TitleEventLogInconsistent = "Inconsistent Event-Log";
    public const string DetailEventLogInconsistent = "The Event-Log is inconsistent.";
    public const string DetailEventLogNotExactlyOneSubmit = "The Event-Log is inconsistent, beacuse it is not excatly one 'submit-submission' contained.";

    public const string TitleAuthenticationTagMissing = "Authentication tags missing";
    public const string TitleAuthenticationTagInvalid = "Authentication tag is invalid";
    public const string DetailAuthenticationTagMissing = "The 'submit-submission'event does not contain authentication tags.";
    public const string DetailAuthenticationTagDataInvalid = "The authentication tag for the submission data is invalid.";
    public const string DetailAuthenticationMetadataInvalid = "The authentication tag for the metadata is invalid.";
    public const string DetailAuthenticationAttachmentInvalid = "The authentication tag for the attachment {0} is invalid.";
    
    public const string TitleDecryptionIssue = "Decryption failed";
    public const string DetailDecryptionIssueMetadata = "Decrypting metadata failed.";
    public const string DetailDecryptionIssueData = "Decrypting submission data failed.";
    public const string DetailDecryptionIssueAttachment = "Decrypting attachment {0} failed.";
    
    public const string TitleMissingData = "Submission data missing";
    public const string DetailMissingData = "Submission data missing.";

    public const string TitleHashMismatch = "Checksum does not match";
    public const string DetailHashMismatchData = "Submission data checksum wrong.";
    public const string DetailHashMismatchAttachment = "Attachment {0} hash value is wrong.";

    public const string TitleMissingSchema = "Schema reference missing";
    public const string DetailMissingSchema = "The schema reference is not conatined in metadata.";

    public const string TitleSchemaViolationData = "Schema violateion";
    public const string DetailsSchemaViolationData = "Submission data does not comply to schema.";

    public const string TitleUnsupportedService = "Service nor supported";
    public const string DetailUnsupportedService = "The given service is not supported";
    
    public const string TitleSyntaxViolation = "Syntax violation";
    public const string DetailSyntaxViolationMetadata = "Metadata is no valid JSON set.";
    public const string DetailSyntaxViolationDataJson = "Submission data is no valid JSON set.";
    public const string DetailSyntaxViolationDataXml = "Submission data is no valid XML set.";
    
    public const string TitleServiceMismatch = "Service type does not match";
    public const string DetailServiceMismatch = "Serivce type of meta data does not match submission.";
    
    
    public const string TitleAttachmentMissing = "List of attachments is invalid";
    public const string DetailAttachmentMissing = "Unable to load attachment {0}";
    
    public const string TitleAttachmentsMismatch = "List of attachments is invalid";
    public const string DetailAttachmentsMismatch = "List of attachments in submssion does not match list on event log.";
    
    public const string TitleUnsupportedSchema = "Schema for meta data is not supported";
    public const string DetailUnsupportedSchema = "Given meta data schema URI does not contain a valid supported meta data schema.";

    public const string TitleUnsupportedDataSchema = "Submission data not supported";
    public const string DetailUnsupportedDataSchema = "Submission data schema is not supported.";
    // @formatter:on

    public enum ProblemTypeEnum {
        AttachmentMismatch,
        MissingAttachments,
        EncryptionIssue,
        HashMismatch,
        IncorrectAuthenticationTag,
        InvalidEventLog,
        MissingAuthenticationTag,
        MissingData,
        MissingSchema,
        ServiceMismatch,
        SchemaViolation,
        SyntaxViolation,
        UnsupportedDataSchema,
        UnsupportedMetaSchema,
        UnsupportedService
    }

    public enum ProblemInstanceEnum {
        Submission,
        Metadata,
        Data
    }

    internal Problems() {
    }

    public Problems(ProblemTypeEnum problemType, string detail) {
        type = "https://schema.fitko.de/fit-connect/events/problems/";
        switch (problemType) {
            case ProblemTypeEnum.MissingAttachments:
                type += "missing-attachment";
                title = TitleAttachmentsMismatch;
                instance = "submission";
                break;
            case ProblemTypeEnum.AttachmentMismatch:
                type += "attachments-mismatch";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.EncryptionIssue:
                type += "encryption-issue";
                title = TitleDecryptionIssue;
                instance = "metadata";
                break;
            case ProblemTypeEnum.HashMismatch:
                type += "hash-mismatch";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.IncorrectAuthenticationTag:
                type += "incorrect-authentication-tag";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.InvalidEventLog:
                type += "invalid-event-log";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.MissingAuthenticationTag:
                type += "missing-authentication-tag";
                title = TitleAuthenticationTagMissing;
                instance = "submission";
                break;
            case ProblemTypeEnum.MissingData:
                type += "missing-data";
                title = TitleMissingData;
                instance = "metadata";
                break;
            case ProblemTypeEnum.MissingSchema:
                type += "missing-schema";
                title = TitleMissingSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.ServiceMismatch:
                type += "service-mismatch";
                title = TitleServiceMismatch;
                instance = "metadata";
                break;
            case ProblemTypeEnum.SchemaViolation:
                type += "schema-violation";
                title = TitleSchemaViolationData;
                instance = "data";
                break;
            case ProblemTypeEnum.SyntaxViolation:
                type += "syntax-violation";
                title = TitleSyntaxViolation;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedDataSchema:
                type += "unsupported-schema";
                title = TitleUnsupportedDataSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedMetaSchema:
                type += "unsupported-schema";
                title = TitleUnsupportedSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedService:
                type += "unsupported-service";
                title = TitleUnsupportedService;
                instance = "metadata";
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(problemType), problemType, null);
        }

        this.detail = detail;
    }


    public Problems(ProblemTypeEnum problemType, string title, string detail,
        string problemInstance) : this(problemType, detail) {
        this.title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        this.instance = problemInstance;
    }
    public Problems(ProblemTypeEnum problemType, string title, string detail,
        ProblemInstanceEnum problemInstance) : this(problemType, detail) {
        this.title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        this.instance = problemInstance switch {
            ProblemInstanceEnum.Metadata => "metadata",
            ProblemInstanceEnum.Submission => "submission",
            ProblemInstanceEnum.Data => "data",
            _ => throw new ArgumentOutOfRangeException(nameof(problemInstance), problemType, null)
        };
    }


    [JsonProperty("instance", NullValueHandling = NullValueHandling.Ignore)]
    public string instance { get; set; }

    [JsonProperty("detail", NullValueHandling = NullValueHandling.Ignore)]
    public string detail { get; set; }

    [JsonProperty("title", NullValueHandling = NullValueHandling.Ignore)]
    public string title { get; set; }

    // [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
    // public string? description { get; set; }

    [JsonProperty("type", NullValueHandling = NullValueHandling.Ignore)]
    public string type { get; set; }

    // [JsonProperty("items", NullValueHandling = NullValueHandling.Ignore)]
    // public Items? items { get; set; }
}
